@@ -2,9 +2,10 @@ module Test-0.0.2;
 use v6;
 
 my $loop = 0;
-my $plan = 0;
+my $plan;
 my $failed = 0;
 my $log_file = %ENV{'TEST_LOG_FILE'};
+my $always_caller = %ENV{'TEST_ALWAYS_CALLER'};
 
 sub plan (Int $number_of_tests) returns Int is export {
     $plan = $number_of_tests;
@@ -25,7 +26,7 @@ sub todo_ok (Bool $cond, Str ?$desc) returns Bool is export {
 ## is
 
 sub is (Str $got, Str $expected, Str ?$desc) returns Bool is export {
-    my $test := $got eq $expected; 
+    my $test := $got eq $expected;
     proclaim($test, $desc, undef, $got, $expected);
 }
 
@@ -34,6 +35,18 @@ sub todo_is (Str $got, Str $expected, Str ?$desc) returns Bool is export {
     proclaim($test, $desc, "TODO", $got, $expected);
 }
 
+## like
+
+sub like (Str $got, Rule $expected, Str ?$desc) returns Bool is export {
+    my $test = $got ~~ $expected;
+    proclaim($test, $desc, undef, $got, $expected);
+}
+
+sub todo_like (Str $got, Rule $expected, Str ?$desc) returns Bool is export {
+    my $test = $got ~~ $expected;
+    proclaim($test, $desc, "TODO", $got, $expected);
+}
+
 ## eval_ok
 
 sub eval_ok (Str $code, Str ?$desc) returns Bool is export {
@@ -90,18 +103,40 @@ sub todo_cmp_ok (Str $got, Code $compare_func, Str $expected, Str ?$desc) return
 
 sub isa_ok ($ref, Str $expected_type, Str ?$desc) returns Bool is export {
     my $ref_type = ref($ref);
-    my $out := defined($desc) ?? $desc :: "The object is-a '$expected_type'";    
+    my $out := defined($desc) ?? $desc :: "The object is-a '$expected_type'";
     my $test := $ref_type eq $expected_type;
     proclaim($test, $out,  undef, $ref_type, $expected_type);
 }
 
 sub todo_isa_ok ($ref, Str $expected_type, Str ?$desc) returns Bool is export {
     my $ref_type = ref($ref);
-    my $out := defined($desc) ?? $desc :: "The object is-a '$expected_type'";         
+    my $out := defined($desc) ?? $desc :: "The object is-a '$expected_type'";
     my $test := $ref_type eq $expected_type;
     proclaim($test, $out, "TODO", $ref_type, $expected_type);
 }
 
+## use_ok
+
+sub use_ok (Str $module) is export {
+    eval "require $module";
+    if ($!) {
+		proclaim(undef, "require $module;", undef, "Import error when loading $module: $!");
+    }
+    else {
+        &ok.goto(1, "$module imported OK");
+    }
+}
+
+sub todo_use_ok (Str $module) is export {
+    eval "require $module";
+    if ($!) {
+		proclaim(undef, "require $module;", undef, "Import error when loading $module: $!");
+    }
+    else {
+        &todo_ok.goto(1, "$module imported OK");
+    }
+}
+
 ## misc. test utilities
 
 sub skip (Str ?$reason) returns Bool is export {
@@ -135,7 +170,8 @@ sub diag (Str $diag) is export {
 
 sub proclaim (Bool $cond, Str ?$desc, Str ?$context, Str ?$got, Str ?$expected) returns Bool {
     my $ok := $cond ?? "ok " :: "not ok ";
-    my $out := defined($desc) ?? " - $desc" :: "";
+    my $out = defined($desc) ?? " - $desc" :: "";
+	$out = "$out <pos:$?CALLER::CALLER::POSITION>" if $always_caller;
     my $context_out := defined($context) ?? " # $context" :: "";
     $loop++;
     say $ok, $loop, $out, $context_out;
@@ -156,7 +192,7 @@ sub report_failure (Str ?$todo, Str ?$got, Str ?$expected) returns Bool {
     }
     if ($?CALLER::CALLER::SUBNAME eq ('&is' | '&todo_is' | '&cmp_ok' | '&todo_cmp_ok' | '&eval_is' | '&todo_eval_is' | '&isa_ok' | '&todo_isa_ok')) {
         diag("  Expected: " ~ ($expected.defined ?? $expected :: "undef"));
-        diag("       Got: " ~ ($got.defined ?? $got :: "undef"));    
+        diag("       Got: " ~ ($got.defined ?? $got :: "undef"));
     }
     else {
         diag("       Got: " ~ ($got.defined ?? $got :: "undef"));
@@ -191,12 +227,10 @@ sub write_log (+$got, +$expected, Str +$desc, Str +$errstr, Str +$context, Str +
 }
 
 END {
-    if (!$plan) {
+    if (!defined($plan)) {
         say("1..$loop");
-    } else {
-        if ($plan != $loop) {
-	    $*ERR.say("# Looks like you planned $plan tests, but ran $loop");
-        }
+    } elsif ($plan != $loop) {
+	$*ERR.say("# Looks like you planned $plan tests, but ran $loop");
     }
     if ($failed) {
         $*ERR.say("# Looks like you failed $failed tests of $loop");
@@ -214,38 +248,43 @@ Test - Test support module for perl6
 
   use v6;
   require Test;
-  
+
   plan 10;
   test_log_file('test.log');
-  
+
+  use_ok('Some::Module');
+  todo_use_ok('Some::Other::Module');
+
   ok(2 + 2 == 4, '2 and 2 make 4');
   is(2 + 2, 4, '2 and 2 make 4');
   isa_ok([1, 2, 3], 'List');
-  
+
   todo_ok(2 + 2 == 5, '2 and 2 make 5');
-  todo_is(2 + 2, 5, '2 and 2 make 5');  
+  todo_is(2 + 2, 5, '2 and 2 make 5');
   todo_isa_ok({'one' => 1}, 'Hash');
-  
+
+  use_ok('My::Module');
+
   pass('This test passed');
   fail('This test failed');
 
   skip('skip this test for now');
-  
+
   todo_fail('this fails, but might work soon');
-  
+
   diag('some misc comments and documentation');
 
 = DESCRIPTION
 
-This module was built to facilitate the Pugs test suite. It has the 
-distinction of being the very first module written for Pugs. 
+This module was built to facilitate the Pugs test suite. It has the
+distinction of being the very first module written for Pugs.
 
-It provides a simple set of common test utility functions, and is 
+It provides a simple set of common test utility functions, and is
 an implementation of the TAP protocol.
 
-This module, like Pugs, is a work in progress. As new features are 
+This module, like Pugs, is a work in progress. As new features are
 added to Pugs, new test functions will be defined to facilitate the
-testing of those features. For more information see the FUTURE PLANS 
+testing of those features. For more information see the FUTURE PLANS
 section of this document.
 
 = FUNCTIONS
@@ -262,17 +301,28 @@ there.  The filename 'test.log' is recommended.
 
 == Testing Functions
 
-- `ok (Bool $cond, Str ?$desc) returns Bool` 
+- `use_ok (Str $module) returns Bool`
+
+*NOTE:* This function currently uses `require()` since Pugs does not yet have
+a proper `use()` builtin.
+
+- `ok (Bool $cond, Str ?$desc) returns Bool`
 
 - `is (Str $got, Str $expected, Str ?$desc) returns Bool`
 
+- `like (Str $got, Rule $expected, Str ?$desc) returns Bool is export`
+
+This function should work with most reg-exps, but given that they are still a
+somewhat experimental feature in Pugs, it is suggested you don't try anything
+too funky.
+
 - `cmp_ok (Str $got, Code $compare_func, Str $expected, Str ?$desc) returns Bool`
 
 This function will compare `$got` and `$expected` using `$compare_func`. This will
 eventually allow Test::More-style cmp_ok() though the following syntax:
 
   cmp_ok('test', &infix:<gt>, 'me', '... testing gt on two strings');
-  
+
 However the `&infix:<gt>` is currently not implemented, so you will have to wait
 a little while. Until then, you can just write your own functions like this:
 
@@ -281,7 +331,7 @@ a little while. Until then, you can just write your own functions like this:
 - `isa_ok ($ref, Str $expected_type, Str ?$desc) returns Bool`
 
 This function currently on checks with ref() since we do not yet have
-object support. Once object support is created, we will add it here, and 
+object support. Once object support is created, we will add it here, and
 maintain backwards compatibility as well.
 
 - `eval_ok (Str $code, Str ?$desc) returns Bool`
@@ -293,15 +343,19 @@ on success, or report that the eval was not successful on failure.
 
 == TODO Testing functions
 
-Sometimes a test is broken because something is not implemented yet. So 
+Sometimes a test is broken because something is not implemented yet. So
 in order to still allow that to be tested, and those tests to knowingly
-fail, we provide a set of todo_* functions for all the basic test 
+fail, we provide a set of todo_* functions for all the basic test
 functions.
 
+- `todo_use_ok(Str $module) returns Bool`
+
 - `todo_ok (Bool $cond, Str ?$desc) returns Bool`
 
 - `todo_is (Str $got, Str $expected, Str ?$desc) returns Bool`
 
+- `todo_like (Str $got, Rule $expected, Str ?$desc) returns Bool is export`
+
 - `todo_cmp_ok (Str $got, Code $compare_func, Str $expected, Str ?$desc) returns Bool`
 
 - `todo_isa_ok ($ref, Str $expected_type, Str ?$desc) returns Bool`
@@ -357,20 +411,15 @@ sub which will be able to handle structures of arbitrary depth and of
 an arbitrary type. The function signatures will likely look something
 like this:
 
-  multi sub is_deeply (Array @got, Array @expected, Str ?$desc) returns Bool;  
-  multi sub is_deeply (List  $got, List  $expected, Str ?$desc) returns Bool;    
-  multi sub is_deeply (Hash  %got, Hash  %expected, Str ?$desc) returns Bool;      
-  multi sub is_deeply (Pair  $got, Pair  $expected, Str ?$desc) returns Bool;  
-  
+  multi sub is_deeply (Array @got, Array @expected, Str ?$desc) returns Bool;
+  multi sub is_deeply (List  $got, List  $expected, Str ?$desc) returns Bool;
+  multi sub is_deeply (Hash  %got, Hash  %expected, Str ?$desc) returns Bool;
+  multi sub is_deeply (Pair  $got, Pair  $expected, Str ?$desc) returns Bool;
+
 Because these functions will be mutually recursive, they will easily be
 able handle arbitrarily complex data structures automatically (at least
 that is what I hope).
 
-- like
-
-This is similar to the Test::More like() function. It will take a
-regular expression reference and compare it with a given string.
-
 - throws_ok, lives_ok
 
 These are functions taken directly from Test::Exception. They will
@@ -379,7 +428,7 @@ or a string to match against the error.
 
 = ENVIRONMENT
 
-Setting the environment variable TEST_LOG_FILE sets the default 
+Setting the environment variable TEST_LOG_FILE sets the default
 filename where test diagnostics should be written.
 
 = SEE ALSO
@@ -413,6 +462,8 @@ Yuval Kogman <nothingmuch@woobling.org>
 
 Nathan Gray <kolibrie@graystudios.org>
 
+Max Maischein <corion@cpan.org>
+
 = COPYRIGHT
 
 Copyright (c) 2005. Autrijus Tang. All rights reserved.
