@@ -1,11 +1,11 @@
-{-# OPTIONS -cpp -fglasgow-exts -fth #-}
+{-# OPTIONS_GHC -fglasgow-exts #-}
 
 module Compile.Pugs where
 import Internals
 
 genPugs :: (Show a, Monad m) => a -> m String
 genPugs exp = return . unlines $
-    [ "{-# OPTIONS -fglasgow-exts -fno-warn-unused-imports -fno-warn-unused-binds -O #-}"
+    [ "{-# OPTIONS_GHC -fglasgow-exts -fno-warn-unused-imports -fno-warn-unused-binds -O #-}"
     , "module MainCC where"
     , "import Run"
     , "import AST"
@@ -13,3 +13,4 @@ genPugs exp = return . unlines $
     , ""
     , "mainCC = runAST $ " ++ show exp
     ]
+
