@@ -1,5 +1,5 @@
-{-# OPTIONS -fglasgow-exts -O #-}
-{-# OPTIONS -#include "UnicodeC.h" #-}
+{-# OPTIONS_GHC -fglasgow-exts -O #-}
+{-# OPTIONS_GHC -#include "UnicodeC.h" #-}
 
 {-
     Lexical analyzer.
@@ -96,8 +96,6 @@ symbol s
     aheadSym '~' y   = not (y `elem` "&|^<>~")
     aheadSym x   y   = y `elem` ";!" || x /= y
 
-stringLiteral = singleQuoted
-
 interpolatingStringLiteral endchar interpolator = do
         list <- stringList
         return $ Syn "cxt" [Val (VStr "Str"), homogenConcat list]
@@ -121,12 +119,6 @@ interpolatingStringLiteral endchar interpolator = do
             return (Val (VStr [char]):rest)
         
 
-singleQuoted = verbatimRule "literal string" $ do
-    str <- between (char '\'') (char '\'' <?> "end of string") (many singleStrChar)
-    return (foldr (id (:)) "" str)
-
-singleStrChar = try quotedQuote <|> noneOf "'"
-
 -- backslahed nonalphanumerics (except for ^) translate into themselves
 escapeCode      = charEsc <|> charNum <|> charAscii <|> charControl <|> anyChar
                 <?> "escape code"
@@ -176,10 +168,6 @@ ascii3          = ['\NUL','\SOH','\STX','\ETX','\EOT','\ENQ','\ACK',
                    '\BEL','\DLE','\DC1','\DC2','\DC3','\DC4','\NAK',
                    '\SYN','\ETB','\CAN','\SUB','\ESC','\DEL']
 
-quotedQuote = do
-    string "\\'"
-    return '\''
-
 rule name action = (<?> name) $ lexeme $ action
 
 verbatimRule name action = (<?> name) $ action
@@ -345,4 +333,4 @@ buildExpressionParser operators simpleExpr
         
       splitOp (Postfix op) (rassoc,lassoc,nassoc,prefix,postfix)
         = (rassoc,lassoc,nassoc,prefix,op:postfix)
-      
+
