@@ -14,9 +14,19 @@ my $data = Load(<$yamlfh>);
 undef $yamlfh;
 #Dump($data);
 
-print "<html><head><link rel='stylesheet' href='testgraph.css' /><title>testgraph.pl ".gmtime()."</title></head><body>\n";
+print <<"__TOP__";
+<!DOCTYPE html
+ PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN'
+ 'http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd'>
+<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='en' lang='en'>
+ <head>
+  <link rel='stylesheet' href='util/testgraph.css' />
+  <title>testgraph.pl @{[gmtime().'']}</title>
+ </head>
+ <body>
+__TOP__
 
-print "<tt><pre>", join("\n", $data->{build_info}), "</pre></tt>\n";
+print "   <pre><tt>", join("\n", $data->{build_info}), "</tt></pre>\n";
 
 print "<table>";
 
@@ -34,21 +44,48 @@ foreach my $testfile (sort {$a->{file} cmp $b->{file}}
   } else {
 	print "<td><table width='100%'><tr>\n";
 	my ($i, $good)=(0, 0);
+
+	my $rows = int(.75 + @{$testfile->{subtests}} / 50) || 1;
+	my $per_row = int(.75 + @{$testfile->{subtests}} / $rows);
+
 	foreach my $test (@{$testfile->{subtests}}) {
 	  my $class = t_to_class($test);
-	  my $title = ($test->{line} || '') . ($test->{diag} || '');
-	  
+
+
+	  my $title = ($test->{line} || '') . "\n" . ($test->{diag} || '');
+
+#	  print STDERR "pre:  $title\n";
+
 	  $title =~ s/\cM//g;
-	  $title =~ s/\cJ$//g;
-	  $title =~ s/([^-().#A-Za-z0-9 ])/sprintf '&#x%X;', ord $1/eg;
-	  
-	  if ($i and $i % 50 == 0) {
-		print "</tr><tr>\n";
+	  $title =~ s/\cJ+$//g;
+	  $title =~ s/^\cJ+//g;
+	  $title =~ s/&/&amp;/g;
+	  $title =~ s/</&lt;/g;
+	  $title =~ s/>/&gt;/g;
+	  $title =~ s!\cJ!<br />!g;
+	  $title =~ s/([^-&<>\/().#A-Za-z0-9 ;])/sprintf '&#x%X;', ord $1/eg;
+
+#	  print STDERR "post: $title\n";
+
+
+	  my $case_link;
+	  ($test->{pos} || '') =~ /^(.*?) at line (\d+), column \d+/;
+	  my ($t, $line) = ($1, $2);
+	  if (defined $line) {
+		$case_link = "$linkto#line_$line";
+	  } else {
+		$case_link = $linkto;
 	  }
-	  
+
+	  if ($i and $i % $per_row == 0) {
+		print "</tr></table><table width='100%'><tr>\n";
+	  }
+
 #	  print "<td class='test $class' title='$title'>$title</td>";
-#	  print " <td class='test $class' title='$title'><a href='$linkto'>$i</a></td>\n";
-	  print " <td class='test $class' title='$title'>&nbsp;</td>\n";
+	  print " <td class='test $class'><a href='$case_link'>&nbsp;<div>$title</div></a></td>\n";
+#	  print STDERR " <td class='test $class'><a href='$case_link'>&nbsp;<div>$title</div></a></td>\n";
+#	  print " <td class='test $class' title='$title'>&nbsp;</td>\n";
+
 	  if ($class ne 'nottest') {
 		$i++;
 		$good++ if $class =~ /good/;
@@ -73,11 +110,11 @@ sub t_to_class {
   my $p;
   my $todo;
   local $_ = $t->{line};
-  
+
   return 'nottest' unless $t->{type} and $t->{type} eq 'test';
-  
+
 #  warn "$_";
-  
+
   if (/^not ok/) {
 	$p=0;
   } elsif (/^ok/) {
@@ -87,7 +124,7 @@ sub t_to_class {
   }
 
   $todo = 0+/# TODO$/;
-  
+
 
   return {
 		  '00'=>'bad',
